(* ------------------------------------------------------ *)
(*                       BDRAW.PAS                        *)
(*          (c) 1990  Ralph Seelig & TOOLBOX              *)
(* ------------------------------------------------------ *)
UNIT BDraw;

INTERFACE (* -------------------------------------------- *)

USES Crt, Graph;

TYPE
  str255 = STRING [255];

CONST
  aktx   : INTEGER = 80;
  akty   : INTEGER = 50;
  aktw   : INTEGER = 0;
  akts   : BYTE    = 4;

  PROCEDURE Draw(s : str255);

IMPLEMENTATION (* --------------------------------------- *)

CONST
  sinfaktor = 0.0174533;

  PROCEDURE Draw(s : str255);
  VAR
    i, x, y,
    code           : INTEGER;
    la, w, x2, y2  : REAL;
    spos           : BYTE;
    hstr           : str255;
    zeichne,
    moving         : BOOLEAN;

    FUNCTION HoleNummer : INTEGER;
    { Holt eine im String enthaltene Zahl und              }
    { interpretiert sie als Integer                        }
    VAR
      hn : INTEGER;
    BEGIN
      Inc(spos);
      REPEAT
        IF s[spos] = ',' THEN Inc(spos);
      UNTIL s[spos] <> ',';
      hstr := Copy(s, spos, 255);
      Val(hstr, hn, code);
      hstr := Copy(s, spos, code - 1);
      spos := spos + code - 2;
      Val(hstr, hn, code);
      HoleNummer := hn;
    END {HoleNummer};

    PROCEDURE DrawLine(wi, la : REAL);
    { Zeichnet eine Linie mit angegebenem Winkel WI        }
    { und der Lnge LA                                     }
    BEGIN
      la := la * (akts / 4);
      y := akty + Round(la * Sin((aktw + wi) * sinfaktor));
      x := aktx + Round(la * Cos((aktw + wi) * sinfaktor));
      IF zeichne THEN Line(aktx, akty, x, y)
                 ELSE zeichne := TRUE;
      IF moving THEN BEGIN
         aktx := x;  akty:= y;
      END;
      moving := TRUE;
    END {DrawLine};

    PROCEDURE MOption;
    BEGIN
      IF ((s[spos+1] = '-') OR (s[spos+1] = '+')) THEN BEGIN
        x2 := HoleNummer;
        y2 := HoleNummer;
        IF x2 = 0 THEN
          x2 := 0.0001;        { Laufzeitfehler verhindern }
        la := Sqrt(Sqr(x2) + Sqr(y2));
        w := ArcTan(y2 / x2) / pi * 180;
        IF x2 < 0 THEN w := 180 + w;
        DrawLine(w, la);
      END ELSE BEGIN
        x := HoleNummer;
        y := HoleNummer;
        IF zeichne THEN Line(aktx, akty, x, y)
                   ELSE zeichne := TRUE;
        IF moving THEN BEGIN
          aktx := x;  akty := y;
        END;
        moving := TRUE;
      END;
    END {MOption};

  BEGIN
    hstr:= 'ABCDEFGHKLMNPRSTU0123456789, +-';
    FOR i := 1 TO Length(s) DO BEGIN
      s[i] := UpCase(s[i]);
      x := Pos(s[i], hstr);
      IF x = 0 THEN BEGIN
        s[i] := 'Z';
        Inc(x);
      END;
    END;
    IF x > 0 THEN
      FOR i := 1 TO x DO BEGIN
        y := Pos('Z', s);
          Delete(s, y, 1);
      END;
    s := Concat(s, ' ');
    zeichne := TRUE;
    moving  := TRUE;
    spos := 1;
    REPEAT
      CASE s[spos] OF
        'B' : zeichne := FALSE;
        'N' : moving  := FALSE;
        'M' : MOption;
        'P' : BEGIN
                x := HoleNummer;
                y := HoleNummer;
                SetFillStyle(1, x);
                FloodFill(aktx, akty, y);
              END;
        'C' : BEGIN
                x := HoleNummer;
                SetColor(x);
              END;
        'A' : aktw := - (HoleNummer MOD 4) * 90;
        'T' : BEGIN
                aktw := HoleNummer;
                IF ((aktw > 359) OR (aktw < -359)) THEN
                  aktw := 0;
                IF aktw < 0 THEN aktw := 360 + aktw;
                aktw := -aktw;
              END;
        'K' : BEGIN
                x := Round(HoleNummer * (akts / 4));
                Circle(aktx, akty, x);
              END;
        'S' : akts := HoleNummer;
        'R' : DrawLine(  0, HoleNummer);
        'D' : DrawLine( 90, HoleNummer);
        'L' : DrawLine(180, HoleNummer);
        'U' : DrawLine(270, HoleNummer);
        'E' : DrawLine(315, HoleNummer);
        'H' : DrawLine(225, HoleNummer);
          'F' : DrawLine( 45, HoleNummer);
        'G' : DrawLine(135, HoleNummer);
      END;
      Inc(spos);
    UNTIL spos > Length(s);
  END {Draw};

BEGIN
END {BDraw}.
(* ------------------------------------------------------ *)
(*                Ende von BDRAW.PAS                      *)
