(* ------------------------------------------------------ *)
(*                      HISTORY.PAS                       *)
(*        Zeichnet die Eingaben am MSDOS-Prompt auf       *)
(*        und speichert sie in einem Ringpuffer zur       *)
(*                 weiteren Verarbeitung                  *)
(*           (C) 1990  Siegfried Hund & TOOLBOX           *)
(* ------------------------------------------------------ *)
(*$R-,S-,I-,V-,B-,N-*)
(*$M 1024, 0, 0*)

PROGRAM History;
USES Dos, Crt;

CONST
  version = 'History V1.0';
  altf10  = $7100;              { Scancode fr <Alt>+<F10> }
  ctrlf10 = $6700;             { Scancode fr <Ctrl>+<F10> }
  return  = $1c0d;                 { Scancode fr <Return> }
  anzahl  = 20;                     { Anzahl Eingabezeilen }

TYPE
  ringpuf = RECORD
              daten : STRING [64];           { Zeilenlnge }
            END;

VAR
  SaveSS, SaveSP,
  ProgSS, ProgSP   : WORD;        { Speicher fr Stacktops }
  SaveInt16        : POINTER;
  ch               : WORD;
  startx, starty,                { Start- und Endeposition }
  endx, endy,                    { der Eingaben am DOS-    }
  col, row         : BYTE;       { PROMPT                  }
  videosegment     : WORD;
  zeiger           : INTEGER;         { Pufferzeilenmerker }
  regs             : Registers;
  zeile            : STRING;
  tdm              : INTEGER;          { Tastendruckmerker }
  ringpuffer       : ARRAY [1..anzahl] OF ringpuf;
                                 { Ringpuffer fr Eingaben }

  FUNCTION GetKey : WORD;
  INLINE($31/$C0/$9C/$FF/$1E/saveint16);

  PROCEDURE ScreenTest;
  BEGIN
    WITH Regs DO BEGIN
      ah := $0F;
      Intr($10, Regs);
      IF al = 7 THEN VideoSegment := $B000     { Monochrom }
                ELSE VideoSegment := $B800;    { Farbe     }
    END;
  END {ScreenTest};

  PROCEDURE CursorPos;
  BEGIN
    Regs.ah := $03;
    Intr($10, Regs);
    col := Succ(Regs.dl);
    row := Succ(Regs.dh);
  END {CursorPos};

  PROCEDURE RingBuffer;
  VAR
    i : INTEGER;
  BEGIN
    FOR i := 1 TO anzahl DO Ringpuffer[i].daten := '';
    zeiger := 1;
    tdm := 0;
  END {RingBuffer};

  PROCEDURE LeseVideo;
  VAR
    displayram : ARRAY [1..2000] OF
                                  WORD ABSOLUTE $B800:$0000;
    stpos, epos, j : WORD;
  BEGIN
    zeile := '';
    stpos := (starty - 1) * 80 + startx;
    epos  := (endy - 1) * 80 + endx;
    FOR j := stpos TO epos DO
      zeile := zeile + Chr(Lo(displayram[j]));
    startx := 0;
    starty := 0;
    endx   := 0;
    endy   := 0;
    IF zeiger > 20 THEN zeiger := 1;
    ringpuffer[zeiger].daten := zeile;
    zeiger := zeiger + 1;
  END {LeseVideo};

  PROCEDURE Anzeigen;
  VAR
    i : INTEGER;
  BEGIN
    WriteLn;
    WriteLn(version);
    FOR i := 1 TO 20 DO BEGIN
      IF i = zeiger - 1 THEN
        Write(' --> ')                   { letzte Eingabe  }
      ELSE
        Write('     ');                  { markieren       }
      WriteLn(ringpuffer[i].daten);
    END;
  END {Anzeigen};

  PROCEDURE DeInstall;
  BEGIN
    WriteLn;
    WriteLn(version, ' wurde reaktiviert ');
    WriteLn;
  END {DeInstall};

(*$F+*)
  PROCEDURE Int16(flags, cs, ip, ax, bx, cx, dx, si,
                         di, ds, es, bp : WORD);  INTERRUPT;

    PROCEDURE ChainInt(address : POINTER);
                           { Sprung zur ISR bei " Address" }
    INLINE($5B/$58/$87/$5E/$0E/$87/$46/$10/$89/
           $EC/$5D/$07/$1F/$5F/$5E/$5A/$59/$CB);

    PROCEDURE SwitchStack;
                      { auf den Programm-Stapel umschalten }
    INLINE($8C/$16/SaveSS/$89/$26/SaveSP/$FA/
           $8E/$16/ProgSS/$8B/$26/ProgSP/$FB);

    PROCEDURE SwitchBack;
                      { vom Programm-Stapel zurckschalten }
    INLINE($FA/$8E/$16/SaveSS/$8B/$26/SaveSP/$FB);

  BEGIN
    IF Hi(ax) = 0 THEN BEGIN           { Tastencode holen? }
      SwitchStack;                     { Umschalten        }
      tdm := tdm + 1;
      IF tdm = 1 THEN BEGIN     { 1. Tastendruck am PROMPT }
        CursorPos;              { Cursorposition ermitteln }
        startx := col;          { und merken               }
        starty := row;
      END;
      ax := GetKey;                     { Tastencode holen }
      ch := ax;                         { und merken       }
      IF ax = altf10 THEN BEGIN         { <Alt>+<F10> ?    }
        Anzeigen;                       { dann anzeigen    }
        ch := return;
      END ELSE
        IF ax = return THEN BEGIN       { <Return> ?       }
          CursorPos;                    { Cursorposition   }
          endx := col;                  { ermitteln und    }
          endy := row;                  { merken           }
          LeseVideo;         { Bildschirmspeicher auslesen }
          tdm := 0;
        END;
      IF ax = ctrlf10 THEN BEGIN        { <Ctrl>+<F10> ?   }
        MemW[$0000:$051E] := $00;       { Speicherzelle:=0 }
        DeInstall;                      { Text ausgeben    }
        SetIntVec($16, SaveInt16);  { alten Int16H setzen  }
        SwitchBack;                 { Stapel-Rckschaltung }
        ax := return;
        ChainInt(SaveInt16);    { Sprung zur alten ISR 16h }
        Halt;
      END;
      SwitchBack;                   { Stapel-Rckschaltung }
      ax := ch;                     { und bergabe         }
    END ELSE
      ChainInt(SaveInt16);      { Sprung zur alten ISR 16h }
  END {Int16};

(*$F-*)

BEGIN
  ProgSS := SSeg;       { Werte fr Programm-Stapel merken }
  ProgSP := SPtr;
  IF Mem[$0000:$051E] <> $55 THEN BEGIN    { Speicherzelle }
    MemW[$0000:$051E] := $55;       { mit Wert beschreiben }
    ScreenTest;                { falls noch nicht resident }
    RingBuffer;
    WriteLn(version);
    WriteLn('Programm installiert');
    WriteLn('ALT +F10 = Anzeige der letzten 20 Eingaben');
    WriteLn('CTRL+F10 = Programm reaktivieren');
  END ELSE BEGIN                    { falls schon resident }
    WriteLn(version);               { Text ausgeben        }
    WriteLn('bereits resident !!');
    Halt;
  END;
  GetIntVec($16, SaveInt16);   { alten Int16-Vektor merken }
  SetIntVec($16, @int16);      { eigene Routine einsetzen  }
  SetIntVec($1B, SaveInt1B);   { BREAK-Vektor zurcksetzen }
  Keep(0);                     { und resident machen       }
END {History}.
(* ------------------------------------------------------ *)
(*                Ende von HISTORY.PAS                    *)
