(* ------------------------------------------------------ *)
(*                    MOUSE.MOD                           *)
(* ------------------------------------------------------ *)
IMPLEMENTATION MODULE Mouse;

FROM SYSTEM  IMPORT ASSEMBLER;
FROM InOut   IMPORT WriteString, Write, Read, WriteCard,
                    WriteLn;
FROM Display IMPORT SetDisplayMode;

CONST
  MaxX      = 640;
  MaxY      = 200;

VAR
  ch        : CHAR;
  mouseinst : CARDINAL;
  col, page : CARDINAL;

  PROCEDURE CheckPos(VAR x,y : CARDINAL);
  (* Position berprfen                                  *)
  BEGIN
    IF y > MaxY THEN y := MaxY;  END;
    IF x > MaxX THEN x := MaxX;  END;
  END CheckPos;

  PROCEDURE InitMouse;
  BEGIN
    ASM
      MOV AX,00H         (* Funktion 0 des Mausinterrupts *)
      INT 33H
    END;
  END InitMouse;

  PROCEDURE ShowMouse;
  BEGIN
    ASM
      MOV AX,01H         (* Funktion 1 des Mausinterrupts *)
      INT 33H
    END;
  END ShowMouse;

  PROCEDURE HideMouse;
  BEGIN
    ASM
      MOV AX,02H         (* Funktion 2 des Mausinterrupts *)
      INT 33H
    END;
  END HideMouse;

  PROCEDURE SetTextMouse(type, start, stop : CARDINAL);
  BEGIN
    ASM
      MOV AX,0AH        (* Funktion 10 des Mausinterrupts *)
      MOV BX,type       (* Software- oder Hardwarecursor  *)
      MOV CX,start      (* Startzeile                     *)
      MOV DX,stop       (* Endzeile                       *)
      INT 33H
    END;
  END SetTextMouse;

  PROCEDURE MousePosition(VAR x, y : CARDINAL);
  VAR
    xpos, ypos : CARDINAL;
  BEGIN
    ASM
      MOV AX,03H         (* Funktion 3 des Mausinterrupts *)
      INT 33H
      MOV xpos,CX        (* Rckgabe der x-Position       *)
      MOV ypos,DX        (* Rckgabe der y-Position       *)
    END;
    x := xpos;
    y := ypos;
  END MousePosition;

  PROCEDURE LeftButton() : BOOLEAN;
  VAR
    tByte : CARDINAL;
  BEGIN
    ASM
      MOV AX,03H         (* Funktion 3 des Mausinterrupts *)
      INT 33H
      MOV tByte,BX       (* Rckgabe der gedrckten Taste *)
    END;
    IF tByte = 1 THEN    (* linke Taste gedrckt          *)
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  END LeftButton;

  PROCEDURE RightButton() : BOOLEAN;
  VAR
    tByte : CARDINAL;
  BEGIN
    ASM
      MOV AX,03H         (* Funktion 3 des Mausinterrupts *)
      INT 33H
      MOV tByte,BX       (* Rckgabe der gedrckten Taste *)
    END;
    IF tByte = 2 THEN    (* rechte Taste gedrckt         *)
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  END RightButton;

  PROCEDURE BothButtons() : BOOLEAN;
  VAR
    tByte : CARDINAL;
  BEGIN
    ASM
      MOV AX,03H        (* Funktion 3 des Mausinterrupts  *)
      INT 33H
      MOV tByte,BX      (* Rckgabe der gedrckten Tasten *)
    END;
    IF tByte = 3 THEN   (* oha, beide Tasten gedrckt...  *)
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  END BothButtons;

  PROCEDURE SetMousePosition(x, y : CARDINAL);
  BEGIN
    CheckPos(x, y);
    ASM
      MOV AX,04H         (* Funktion 4 des Mausinterrupts *)
      MOV CX,x           (* x-Position bergeben          *)
      MOV DX,y           (* y-Position bergeben          *)
      INT 33H
    END;
  END SetMousePosition;

  PROCEDURE MouseInArea(x1, y1, x2, y2 : CARDINAL): BOOLEAN;
  VAR
    x, y : CARDINAL;
    X, Y : CARDINAL;
  BEGIN
    MousePosition(X, Y);
    IF ((X >= x1) AND (X <= x2)) AND
       ((Y >= y1) AND (Y <=y2)) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  END MouseInArea;

  PROCEDURE VerticalBorder(left, right : CARDINAL);
  BEGIN
    ASM
      MOV AX,07H         (* Funktion 7 des Mausinterrupts *)
      MOV CX,left        (* linke Grenze bergeben        *)
      MOV DX,right       (* rechte Grenze bergeben       *)
      INT 33H
    END;
  END VerticalBorder;

  PROCEDURE HorizontalBorder(top, bottom : CARDINAL);
  BEGIN
    ASM
      MOV AX,08H         (* Funktion 8 des Mausinterrupts *)
      MOV CX,top         (* obere Grenze bergeben        *)
      MOV DX,bottom      (* untere Grenze bergeben       *)
      INT 33H
    END;
  END HorizontalBorder;

BEGIN
  SetDisplayMode(3);     (* Textmodus, und dann:          *)
  InitMouse;             (* Die Sendung fr die Maus...   *)
END Mouse.
(* ------------------------------------------------------ *)
(*                 Ende von MOUSE.MOD                     *)
